/**/

#ifndef _EXECUTOR_H
#define _EXECUTOR_H

#include <gio/gio.h>
#include <glib/gstdio.h>
#include <signal.h>
#include <errno.h>

G_BEGIN_DECLS

#define INFO_EXECUTE "execute"
#define INFO_STDOUT_STRINGS "stdout_strings"
#define INFO_STDOUT_BYTES "stdout_bytes"
#define INFO_STDERR_STRINGS "stderr_strings"
#define INFO_STDOUT_SIGNAL_NAME "stdout_signal_name"
#define INFO_STDERR_SIGNAL_NAME "stderr_signal_name"
#define INFO_STDOUT_BYTE_LIMIT "stdout_byte_limit"
#define INFO_STDOUT_STRINGS_LIMIT "stdout_strings_limit"
#define INFO_STDERR_STRINGS_LIMIT "stderr_strings_limit"
#define INFO_THREAD_LIMIT "thread_limit"
#define INFO_ACTION_ID "action_id"
#define INFO_TIMEOUT "timeout"
#define THREAD_LIMIT 1
#define PLUGIN_NAME "executor"
#define DBUS_NAME "ru.basealt.alterator.executor"
#define MAX_NAME_LENGTH 100
#define ENABLED "enabled"
#define DISABLED "disabled"
#define TWO_ARRAYS "(asasi)"
#define BYTES_AND_ARRAY "(ayasi)"
#define ARRAY_ONLY "(asi)"
#define BYTES_ONLY "(ayi)"
#define EXIT_STATUS_ONLY "(i)"
#define DEFAULT_BYTE_LIMIT 524288
#define DEFAULT_TIMEOUT 60

typedef struct {
    gchar     *method_name;
    gchar     *execute;
    gchar     *stdout_signal_name;
    gchar     *stderr_signal_name;
    gchar     *action_id;
    gchar    **data;
    gboolean   stdout_strings_enabled;
    gboolean   stdout_bytes_enabled;
    gboolean   stdout_signals_enabled;
    gboolean   stderr_strings_enabled;
    gboolean   stderr_signals_enabled;
    gint       stdout_strings_limit;
    gint       stdout_byte_limit;
    gint       stderr_strings_limit;
    gint       thread_limit;
    gint       thread_counter;
    gint       timeout;
} MethodInfo;

typedef struct {
    gint         thread_limit;
    gint         thread_counter;
    GHashTable   *methods;
} InterfaceInfo;

typedef struct {
    gchar                 *command_line;
    GDBusMethodInvocation *invocation;
    GDBusConnection       *connection;
    gchar                 *sender;
    gchar                **envp; //environment
    InterfaceInfo         *interface_info; //It doesn't need to be freed,
    MethodInfo            *method_info;    //it belongs to methods_table.
    const gchar           *object_path;
    const gchar           *interface_name;
    gchar                 *stdout_signal_name;
    gchar                 *stderr_signal_name;
    gint                   stdout_strings_limit;
    gint                   stdout_byte_limit;
    gint                   stderr_strings_limit;
    gint                   stdout_strings_counter;
    gint                   stdout_byte_counter;
    gint                   stderr_strings_counter;
    /* Enable or disable returning out/err as a string array or byte array (only
       stdout), or line by line via signals. */
    gboolean               stdout_strings_enabled;
    gboolean               stdout_bytes_enabled;
    gboolean               stdout_signals_enabled;
    gboolean               stderr_strings_enabled;
    gboolean               stderr_signals_enabled;
    GList                 *stdout_strings;
    GList                 *stderr_strings;
    GString               *stdout_bytes;
    /* These fields is used to signal buffer overruns (stdout/stderr) and the
       like. */
    gint                   stdout_exit_status;
    gint                   stderr_exit_status;
    gboolean               exit_status;
} ThreadFuncParam;

typedef struct {
    GMainLoop       *loop;
    gboolean         child_exited;
    gboolean         stdout_done;
    gboolean         stderr_done;
    ThreadFuncParam *thread_func_param;
    gint             stdout_pipe_out;
    gint             stderr_pipe_out;
} ChildData;

G_END_DECLS

#endif
